@@ -4,7 +4,7 @@ use 5.006;
 use strict;
 use warnings;
 
-use List::Util      qw/min/;
+use List::Util      qw/any min/;
 use List::MoreUtils qw/zip first_index/;
 use Scalar::Util    qw/looks_like_number/;
 
@@ -14,11 +14,11 @@ Games::Cards::ShuffleTrack - Track cards through shuffles and cuts
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 my $cut_limits = {
 	normal  => [0.19, 0.82], # on a 52 cards deck, cut between 10 and 43 cards
@@ -59,7 +59,7 @@ This module allows you to simulate true and false shuffles and cuts.
 	$deck->riffle_shuffle();
 	$deck->cut( 'short' );
 	$deck->riffle_shuffle();
-	print $deck->get_deck();
+	print "@{$deck->get_deck()}";
 
 Or perhaps with more precision:
 
@@ -69,7 +69,7 @@ Or perhaps with more precision:
 	$deck->cut( 26 );
 	print $deck->get_deck();
 
-See the rest of the documentation for more advanced features.
+See the rest of the documentation for more advanced features. See the examples folder for more detailed usage.
 
 
 =head1 INSTALLATION
@@ -153,7 +153,7 @@ Reset the deck to its original status. The original is whatever you selected whe
 
 	$deck->restart;
 
-Do bear in mind that by doing so you are replinishing the deck of whatever cards you took out of it.
+Do bear in mind that by doing so you are replenishing the deck of whatever cards you took out of it.
 
 =cut
 
@@ -167,9 +167,7 @@ sub restart {
 
 Returns the size of the deck.
 
-    for ( 1 .. $deck->deck_size ) {
-        ...
-    }
+    my $size = $deck->deck_size;
 
 =cut
 
@@ -494,6 +492,7 @@ For more information on how to cut to a specific card please refer to the L<SEE
 
 =cut
 
+# TODO: delimit randomness of the cut between two numbers: $deck->cut( 1, 25 );
 sub cut {
 	my $self     = shift;
 	my $position = shift;
@@ -527,6 +526,108 @@ sub cut_above {
 	return $self->cut( $self->find( $card ) - 1 );
 }
 
+=head3 cut_to
+
+Cuts a portion of the deck to another position
+
+	$deck->cut_to( $pile );
+
+You can specify exactly how many cards to cut or delimit the randomness of the cut:
+
+	# cut exactly 15 cards to $pile
+	$deck->cut_to( $pile, 15 );
+
+	# cut between 10 and 26 cards to $pile
+	$deck->cut_to( $pile, 10, 26 );
+
+If the position doesn't exist yet you can also automatically create it:
+
+	my $pile = $deck->cut_to();
+
+This method returns the new pile.
+
+=cut
+
+sub cut_to {
+	my $self        = shift;
+
+	# create the new pile if required
+	my $new_pile;
+	if ( defined($_[0]) and ref( $_[0] ) eq 'Games::Cards::ShuffleTrack' ) {
+		$new_pile = shift;
+	}
+	else {
+		$new_pile = Games::Cards::ShuffleTrack->new( 'empty' );
+	}
+
+	# set the position
+	my $lower_limit = shift;
+	my $upper_limit = shift;
+
+	my $position;
+
+	if ( not defined $lower_limit ) {
+		$position = _rand( 1, $self->deck_size );
+	}
+	elsif ( not defined $upper_limit ) {
+		$position = $lower_limit;
+	}
+	else {
+		$position = _rand( $lower_limit, $upper_limit );
+	}
+
+	# cut the deck
+	$new_pile->place_on_top( splice @{$self->get_deck}, 0, $position );
+
+	return $new_pile;
+}
+
+
+=head3 place_on_top
+
+Places a pile of cards on top of the deck.
+
+	$deck->place_on_top( qw/AS KS QS JS 10S/ );
+
+=cut
+
+sub place_on_top {
+	my $self = shift;
+	my @pile = @_;
+
+	$self->_set_deck( @pile, @{$self->get_deck} );
+
+	return $self;
+}
+
+=head3 complete_cut, move_to
+
+Complete the cut by moving all cards from one deck onto another:
+
+	$deck->complete_cut( $new_pile );
+	
+	# or
+
+	$deck->move_to( $table );
+
+=cut
+
+sub complete_cut {
+	my $self = shift;
+	my $destination = shift;
+
+	$self->cut_to( $destination, $self->deck_size );
+
+	return $self;
+}
+
+sub move_to {
+	my $self = shift;
+
+	return $self->complete_cut( @_ );
+}
+
+
 =head3 running_cuts
 
 Cut packets:
@@ -715,6 +816,40 @@ sub put {
 }
 
 
+=head3 insert
+
+Inserts a card in a specified position in the deck. If the position isn't specified than the card is inserted somewhere at random.
+
+	# insert a Joker at position 20
+	$deck->insert( 'Joker', 20 );
+
+	# replace a card somewhere in the deck at random
+	$deck->insert( $card );
+
+If the position doesn't exist the card will be replaced at the bottom of the deck.
+
+=cut
+
+# TODO: inserting in negative position?
+# TODO: what if the user inserts at position 0?
+sub insert {
+	my $self     = shift;
+	my $card     = shift;
+	my $position = shift;
+
+	if ( not defined $position ) {
+		$position = _rand( 1, $self->deck_size );
+	}
+	elsif ( $position > $self->deck_size ) {
+		$position = $self->deck_size + 1;
+	}
+
+	splice @{$self->get_deck}, $position - 1, 0, $card;
+
+	return $self;
+}
+
+
 =head3 deal
 
 Deals a card, removing it from the deck.
@@ -756,7 +891,7 @@ Removes a card from the deck.
 =cut
 
 sub remove {
-	my $self = shift;
+	my $self     = shift;
 	my $position = shift;
 
 	my @deck = @{$self->get_deck};
@@ -851,7 +986,7 @@ sub _cut_depth {
 		$position = 'normal';
 	}
 
-	if ($position =~ /^short|center|deep|normal$/) {
+	if ( any { $_ eq $position } keys %$cut_limits ) {
 		my ($lower, $upper) = @{$cut_limits->{ $position }};
 		$position = _rand( $deck_size * $lower, $deck_size * $upper );
 	}
